<?php

function uc_batch_price_default_form(){
  $enabled = variable_get('uc_price_per_role_enabled', array());
  $enabled_roles = array_filter($enabled);
  $uc_price_per_role_default = variable_get('uc_price_per_role_default', array());
  $roles = user_roles();
  //$rid = 1;
  //debug($roles);
  foreach($enabled_roles as $rid => $enabled){
	//$rid++;
    $form['role'][$rid] = array(
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#title' => $roles[$rid],
    );
    $form['role'][$rid]['multiplicator_'.$rid] = array(
      '#type' => 'textfield',
      '#title' => t('乘数'),
      '#description' => t('大于0小于等于1的小数'),
      '#default_value' => isset($uc_price_per_role_default[$rid]) ? $uc_price_per_role_default[$rid] : 0.8,
      '#size' => 16,
    );
  }
  $form['submit'] = array(
    '#type' =>'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

function uc_batch_price_default_form_validate($form, &$form_state) {
  $enabled = variable_get('uc_price_per_role_enabled', array());
  $enabled_roles = array_filter($enabled);
  foreach($enabled_roles as $rid => $enabled){
    $multiplicator = $form_state['values']['multiplicator_'.$rid];
    if (!empty($multiplicator) && !is_numeric($multiplicator)) {
      form_set_error('multiplicator_'.$rid, t('乘数必须是一个数字.'));
    }
    elseif (!empty($multiplicator)) {
      if(($multiplicator > 1) || ($multiplicator < 0)){
         form_set_error('multiplicator_'.$rid, t('乘数必须是一个大于0小于等于1的数字.'));
	  }
    }
  }	
}

function uc_batch_price_default_form_submit($form, &$form_state) {
  $enabled = variable_get('uc_price_per_role_enabled', array());
  $enabled_roles = array_filter($enabled);
  $uc_price_per_role_default = array();
  foreach($enabled_roles as $rid => $enabled){
    $uc_price_per_role_default[$rid] = $form_state['values']['multiplicator_'.$rid];
  }
  variable_set('uc_price_per_role_default', $uc_price_per_role_default);
}

function uc_batch_price_batch_form(){
  $product_types = uc_product_types();
  $types = node_type_get_types();	
  $options = array();
  //debug($product_types);
  //debug($types);
  foreach ($product_types as $key => $value) {
    $options[$value] = $types[$value]->name;
  }
	
  $form['types'] = array(
    '#title' => t('Content types to be processed'),
    '#type' =>'checkboxes',
    '#description' =>t('All nodes of these selected types will be processed'),
    '#options' => $options,
  );
  $form['submit'] = array(
    '#type' =>'submit',
    '#value' => t('Submit'),
  );
  return $form;
}


function uc_batch_price_batch_form_submit($form,&$form_state){
  $types = array_filter($form_state['values']['types']);
  //drupal_set_message('1234567');
  $batch = array(
    'operations' => array(
      array('uc_batch_price_price_process',array($types)),
    ),
    'finished' => 'uc_batch_price_batch_finished',
    'title' => t('批量处理价格'),
    'init_message' => t('开始批量处理价格.'),
    //'progress_message' => t('Reindexed @current out of @total.'),
    'error_message' => t('批量处理价格遇到错误.'),
	'file' => drupal_get_path('module', 'uc_batch_price') . '/uc_batch_price.admin.inc',
  );
  batch_set($batch);
}

function uc_batch_price_price_process($types, &$context){
  //drupal_set_message('123456');
  //debug($types);
  $size = 20;
  //drupal_set_message('123456');
  if(!isset($context['sandbox']['progress'])){
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['last_nid'] = 0;
    $context['sandbox']['max'] = db_select('node', 'n')
      ->condition('n.type', $types, 'IN')
      ->countQuery()
      ->execute()
      ->fetchField();
  }
  $enabled = variable_get('uc_price_per_role_enabled', array());
  $enabled_roles = array_filter($enabled);
  $uc_price_per_role_default = variable_get('uc_price_per_role_default', array());
  //drupal_set_message('123456');
  $query = db_select('node', 'n')
    ->fields('n', array('nid'))
    ->condition('n.type', $types, 'IN')
	->condition('n.nid', $context['sandbox']['last_nid'], '>')
	->orderBy('n.nid', 'ASC')
    ->range(0, $size);
  $result = $query->execute();
  foreach ($result as $record) {
    //drupal_set_message('123456');
    $product = node_load($record->nid);
	$list_price = $product->list_price;
    foreach($enabled_roles as $rid => $enabled){
      if(!empty($uc_price_per_role_default[$rid])){
      //if(empty($product->role_prices[$rid]) && !empty($uc_price_per_role_default[$rid])){
	    $product->role_prices[$rid] = $list_price * $uc_price_per_role_default[$rid];
	  }
	}	
    node_save($product);
    $context['sandbox']['progress']++;
    $context['sandbox']['last_nid'] = $product->nid;
  }
  if($context['sandbox']['progress'] ==$context['sandbox']['max']){
    $context['finished'] = 1;
  }else{
    $context['finished'] = $context['sandbox']['progress']/$context['sandbox']['max'];
  }
}

function uc_batch_price_batch_finished($success, $results, $operations){
  if ($success) {
    // Here we do something meaningful with the results.
    $message = t('批量处理价格完成');	
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = '在批量处理价格时出现一个错误'. $error_operation[0] .' 其参数为 :'. print_r($error_operation[0], TRUE);
  }
  drupal_set_message($message);
}